// SPDX-License-Identifier: GPL-2.0
/*-
 * Copyright(c) 2020-21 Intel Corporation. All rights reserved.
 */
#include <linux/kernel.h>
#include <linux/device.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/list.h>
#include <linux/dma-mapping.h>
#include <linux/dmapool.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/mutex.h>
#include <linux/aio.h>
#include <linux/uio.h>
#include <linux/wait.h>
#include <linux/poll.h>
#include <linux/irqreturn.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/mm_types.h>
#include <asm/io.h>
#include <linux/eventfd.h>
#include <linux/version.h>
#include <linux/highmem.h>
#include "ifc_mcdma_chr.h"
#include <mcdma_ip_params.h>
#include <regs/qdma_regs_2_registers.h>
#include <asm/page.h>

#define ifc_wmb()		__asm volatile("sfence" ::: "memory")
#define IFC_CNT_HEAD_MOVE	100000

#ifdef IFC_QDMA_MSI_ENABLE
#define IFC_MCDMA_BAS_EN
#define HAVE_ALLOC_IRQ_VECTORS
#endif

static DECLARE_WAIT_QUEUE_HEAD(ifc_device_wait);
static enum ifc_config_mcdma_cmpl_proc config_mcdma_cmpl_proc =
					IFC_CONFIG_MCDMA_COMPL_PROC;


static int ifc_descq_processed_count(struct ifc_mcdma_queue *q, int *cur_head);

#ifdef IFC_QDMA_INTF_AVST
static int ifc_check_payload(int payload)
{
	int new_payload = 0;
	int remainder = payload % 64;

	if (remainder == 0)
		return payload;

	/* Rounding up to the nearest multiple of a 64 */
	new_payload = payload + 64 - remainder;

	return new_payload;
}
#endif

static int ifc_mem_test(void *addr, size_t len, uint32_t pattern)
{
	uint32_t *ptr = (uint32_t *)addr;
	int i;

	for (i = 0; i < len / sizeof(uint32_t); i++)
		ptr[i] = pattern;

	for (i = 0; i < len / sizeof(uint32_t); i++) {
		if (ptr[i] != pattern)
			return -1;
	}
	return 0;
}

#ifdef DEBUG
/**
 * ifc_mcdma_print_qstats - display queue statistics
 */
static void ifc_mcdma_print_qstats(struct ifc_mcdma_queue *q) {

        if (unlikely(q == NULL || q->chnl->channel_id > IFC_MCDMA_CHANNEL_MAX)) {
		pr_err("Invalid channel context\n");
                return;
        }

        printk("SW Stats: chnl:%u dir:%s, ", q->chnl->channel_id, q->dir == IFC_MCDMA_DIR_RX ? "Rx" : "Tx");
        printk("processed head:%u, ", q->cur_con_head);
        printk("WB head :%u\n", (volatile uint32_t)q->consumed_head);
        printk("SW head :%u, ", q->head);
        printk("SW tail :%u, ", q->tail);
        printk("DIDX:%u, ", q->didx);
        printk("HW head:%u, ", readl(q->qcsr + QDMA_REGS_2_Q_HEAD_POINTER));
        printk("WB reg:%u\n", readl(q->qcsr + QDMA_REGS_2_Q_COMPLETED_POINTER));
}
#endif

static int
ifc_dma_mem_alloc(struct dma_pool *dmem_pool, size_t dmem_pool_size, void **mem_virt_addr,
		  dma_addr_t *mem_dma_addr)
{
	if (unlikely(dmem_pool == NULL)) {
		pr_err("Invalid DMA Pool Context\n");
		return -EFAULT;
	}

	*mem_virt_addr = dma_pool_alloc(dmem_pool, GFP_KERNEL, mem_dma_addr);
	if (*mem_virt_addr == NULL) {
		pr_err("DMA mem allocation failed\n");
		return -ENOMEM;
	}

	if (ifc_mem_test(*mem_virt_addr, dmem_pool_size, DRIVER_PATTERN)) {
		pr_err("Memtest failed\n");
		dma_pool_free(dmem_pool, *mem_virt_addr, *mem_dma_addr);
		return -EAGAIN;
	}
	return 0;
}

#ifdef IFC_MCDMA_KMEM
static void
ifc_dma_mem_free(struct device *dev, struct dma_pool *dmem_pool,
		 struct list_head *mem_list)
{
	struct ifc_dma_mem_info *mem_info, *n;

	if (unlikely(dev == NULL || dmem_pool == NULL || mem_list == NULL)) {
		pr_err("Invalid DMA Pool Context\n");
		return;
	}

	list_for_each_entry_safe(mem_info, n, mem_list, list) {
		dma_pool_free(dmem_pool, mem_info->mem_virt_addr,
				mem_info->mem_dma_addr);
		list_del(&mem_info->list);
		kfree(mem_info);
	}
	pr_debug("All DMA mem dealloced\n");
}
#endif /* IFC_MCDMA_KMEM */

static int q_size_bits(uint32_t val)
{
	int i;

	for (i = 8 * sizeof(uint32_t) - 1; i >= 0; i--) {
		if (val >> i)
			return i;
	}
	return -1;
}

#ifdef IFC_MCDMA_KMEM
#ifdef IFC_QDMA_INTF_AVST
static int ifc_meta_data_handler(struct ifc_mcdma_queue *q, int nr)
{
	struct ifc_mcdma_desc_sw *sw;
	struct ifc_mcdma_desc *desc;
	struct ifc_dma_mem_info *mem_info;
	struct request_struct *req;
	int i = 0;
	int t;
	uint32_t sof = 0, eof = 0;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	desc = (struct ifc_mcdma_desc *)
		(q->ring_virt_addr + sizeof(*desc) * q->head);

	for (t = q->head; i < nr; t = (t + 1) % q->qlen) {
		if (desc->link) {
			if (desc->src == q->ring_phy_addr)
				desc =
				(struct ifc_mcdma_desc *)(q->ring_virt_addr);
			else
				desc++;
			--nr;
			continue;
		}
		sw = (struct ifc_mcdma_desc_sw *)&q->ctx[t];
		mem_info = (struct ifc_dma_mem_info *)sw->ctx_data;
		if (mem_info == NULL) {
			pr_err("Something bad happended with data structure, descq ring index: %d, nr: %d\n", t, nr);
			break;
		}

		req = (struct request_struct *)(mem_info->mem_virt_addr);
		sof = desc->sof;
		eof = desc->eof;

		/* sof eof info handling */
		if (eof) {
			req->flags |= IFC_MCDMA_EOF_MASK;
			req->pyld_cnt = desc->rx_pyld_cnt;
			/* pyld_cnt 0 is the special value.
			 * Represents 1MB
			 */
			if (desc->rx_pyld_cnt == 0)
				req->pyld_cnt = 1 << 20;
		}
		if (sof)
			req->flags |= IFC_MCDMA_SOF_MASK;
		i++;
		desc++;
	}
	return 0;
}
#endif
#endif /* IFC_MCDMA_KMEM */

static int ifc_queue_poll(struct ifc_mcdma_queue *q)
{
	int desc_per_page;
	int links;
	int head;
#ifdef IFC_MCDMA_KMEM
#ifdef IFC_QDMA_INTF_AVST
	int ret;
#endif
#endif
	int nr;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

#ifdef IFC_MCDMA_BAS_EN
	return 1;
#endif
	desc_per_page = PAGE_SIZE / sizeof(struct ifc_mcdma_desc);
	nr = ifc_descq_processed_count(q, &head);
	q->head_didx = q->consumed_head;

#ifdef IFC_MCDMA_KMEM
#ifdef IFC_QDMA_INTF_AVST
	/* load meta-data if H2D */
	if (q->dir == IFC_MCDMA_DIR_RX) {
		ret = ifc_meta_data_handler(q, nr);
		if (ret < 0)
			return ret;
	}
#endif
#endif

	links = (head / desc_per_page - q->head / desc_per_page);

	if (head > q->head)
		nr -= links;
	else if (head < q->head)
		nr -= (q->num_desc_pages + links);
	else if (head == q->head && nr)
		nr -= q->num_desc_pages;

	q->head = head;
	return nr;
}

static irqreturn_t ifc_mcdma_irq_handler(int irq, void *arg)
{
	struct ifc_mcdma_queue *q;
	int nr;

	if (arg == NULL) {
		pr_err("IRQ Handler: Failed to retrive queue context\n");
		return IRQ_NONE;
	}

	q = (struct ifc_mcdma_queue *)arg;
	nr = ifc_queue_poll(q);
	if (q->ectx && nr > 0)
		eventfd_signal(q->ectx, nr);

	return IRQ_HANDLED;
}

static int ifc_irq_handler_alloc(struct ifc_msix_info *msix_info,
				struct ifc_mcdma_queue *q)
{
	unsigned long irq_flags = (IRQF_NO_THREAD | IRQF_NOBALANCING |
				   IRQF_ONESHOT | IRQF_IRQPOLL);
	int irq_index;
	int ret;

	if (unlikely(q == NULL || msix_info == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	irq_index = (q->chnl->channel_id * 4) + ((1 - q->dir) * 2);

	/* register irq handler */
	ret = request_irq(msix_info->table[irq_index].vector,
			  ifc_mcdma_irq_handler, irq_flags,
			  IFC_MCDMA_DRIVER_NAME, q);
	if (ret < 0) {
		pr_err("Failed to register irq handler: %d\n", ret);
		return -1;
	}
	q->irq_index = irq_index;
	pr_debug("Successfully registered irq handler: nvectors: %d, irq_index: %d\n",
		msix_info->nvectors, irq_index);
	return 0;
}

#ifdef IFC_MCDMA_BAS_EN
/* MSI register for BAS */
//static int ifc_msi_irq_handler_alloc(struct ifc_mcdma_queue *q,
static int ifc_msi_irq_handler_alloc(struct ifc_mcdma_channel *chnl_ctx, struct ifc_mcdma_queue *q,
			struct ifc_msi_info *msi_info, int vec)
{
	unsigned long irq_flags = (IRQF_NO_THREAD | IRQF_NOBALANCING |
				   IRQF_ONESHOT | IRQF_IRQPOLL);
	int ret;
	int irq;
	struct ifc_pci_dev *pci_ctx;

	if (msi_info == NULL) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	pci_ctx = &(chnl_ctx->dev_ctx->pdev_ctx->pctx);
	irq = pci_irq_vector(pci_ctx->pdev, vec);

	/* register irq handler */
	ret = request_irq(irq,
			  ifc_mcdma_irq_handler, irq_flags,
			  IFC_MCDMA_DRIVER_NAME, q);
	if (ret < 0) {
		pr_err("Failed to register irq handler: %d\n", ret);
		q->irq_index = -1;
		chnl_ctx->direction = -1;
		q->irq = -1;
		return -1;
	}
	q->irq = irq;
	chnl_ctx->direction = vec;
	q->irq_index = vec;
	q->dir = vec;

	pr_err("Successfully registered irq handler: nvectors: %d, irq_index: %d\n",
		msi_info->nvectors, q->irq_index);
	return 0;
}

static void ifc_release_msi_interrupts(struct ifc_mcdma_channel *chnl_ctx)
{
	struct ifc_pci_dev *pci_ctx;

	pci_ctx = &(chnl_ctx->dev_ctx->pdev_ctx->pctx);

	if(chnl_ctx->direction == 0){
		free_irq(chnl_ctx->rx.irq, &(chnl_ctx->rx));
	}
	else if(chnl_ctx->direction == 1){
		free_irq(chnl_ctx->tx.irq, &(chnl_ctx->tx));
	}
	kfree(chnl_ctx);
}
#endif

static void
ifc_irq_handler_free(struct ifc_msix_info *msix_info,
		     struct ifc_mcdma_queue *q)
{
	int irq_index;

	if (unlikely(q == NULL || msix_info == NULL)) {
		pr_err("Invalid Queue Context\n");
		return;
	}

	irq_index = q->irq_index;

	free_irq(msix_info->table[irq_index].vector, q);
	pr_debug("Successfully freed irq handler: nvectors: %d, irq_index: %d\n",
		msix_info->nvectors, q->irq_index);
}

static int ifc_mcdma_descq_init(struct ifc_mcdma_queue *q)
{
	QDMA_REGS_2_Q_CTRL_t q_ctrl = { { 0 } };
	struct ifc_pci_dev *pci_ctx;
	struct ifc_mcdma_desc *desc;
	struct ifc_mcdma_desc_sw *sw;
	int desc_per_page;
	uint64_t cons_head;
	uint32_t log_len;
	int ret;
	int i;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	desc_per_page = PAGE_SIZE / sizeof(struct ifc_mcdma_desc);

	q->qlen = q->num_desc_pages * desc_per_page;

	/* allocate the ring buffer */
	pci_ctx = &q->chnl->dev_ctx->pdev_ctx->pctx;
	q->ring_virt_addr = dma_alloc_coherent(&pci_ctx->pdev->dev,
			q->num_desc_pages * PAGE_SIZE,
			&q->ring_dma_addr, GFP_KERNEL);
	if (q->ring_virt_addr == NULL)
		return -ENOMEM;

	log_len = q_size_bits(q->qlen);
	q->num_desc_bits = log_len;
	q->ring_phy_addr = virt_to_phys(q->ring_virt_addr);

	/* allocate memory to hold onto the
	 * requests passed from the application
	 */
	q->ctx = (struct ifc_mcdma_desc_sw *)
		kcalloc(q->qlen, sizeof(*sw), GFP_KERNEL);
	if (q->ctx == NULL)
		goto fail_cleanup;

	/* link the descriptor pages */
	desc = (struct ifc_mcdma_desc *)q->ring_virt_addr;
	for (i = 0; i < q->qlen; i++)
		desc[i].link = 0;

	desc[i - 1].src = q->ring_phy_addr;
	desc[i - 1].link = 1;

	for (i = desc_per_page; i < q->qlen; i += desc_per_page) {
		desc[i - 1].link = 1;
		desc[i - 1].src = virt_to_phys(desc + i);
	}

	/* program regs */
	writel((uint32_t)q->ring_phy_addr,
		q->qcsr + QDMA_REGS_2_Q_START_ADDR_L);
	writel(q->ring_phy_addr >> 32, q->qcsr + QDMA_REGS_2_Q_START_ADDR_H);
	writel(log_len, q->qcsr + QDMA_REGS_2_Q_SIZE);
	writel(0U, q->qcsr + QDMA_REGS_2_Q_TAIL_POINTER);

	/* configure address for write backs from HW */
	cons_head = (uint64_t)virt_to_phys(&(q->consumed_head));
	writel((uint32_t)cons_head,
		q->qcsr + QDMA_REGS_2_Q_CONSUMED_HEAD_ADDR_L);
	writel(cons_head >> 32, q->qcsr + QDMA_REGS_2_Q_CONSUMED_HEAD_ADDR_H);

	/* configure batch delay */
	writel(IFC_MCDMA_Q_BATCH_DELAY, q->qcsr + QDMA_REGS_2_Q_BATCH_DELAY);

#ifdef IFC_QDMA_INTF_AVST
	/* configure payload */
	writel(0, q->qcsr + QDMA_REGS_2_Q_DEBUG_STATUS_4);
#endif
	/* configure the control register */
	q_ctrl.field.q_en = 1;

	switch (config_mcdma_cmpl_proc) {
	case CONFIG_MCDMA_QUEUE_WB:
		q_ctrl.field.q_wb_en = 1;
		q->wbe = 1;
		q->ire = 0;
		break;
	case CONFIG_MCDMA_QUEUE_REG:
		pr_err("Register Read Completion Notification Policy not allowed\n");
		return -1;
	case CONFIG_MCDMA_QUEUE_MSIX:
		q_ctrl.field.q_intr_en = 1;
		q_ctrl.field.q_wb_en = 1;
		q->ire = 1;
		q->wbe = 0;
		/* register irq handler */
		ret = ifc_irq_handler_alloc(&q->chnl->dev_ctx->pdev_ctx->msix_info, q);
		if (ret < 0)
			goto fail_cleanup;

		break;
	default:
		goto fail_cleanup;
	}

	/* write in QCSR register */
	writel(q_ctrl.val, q->qcsr + QDMA_REGS_2_Q_CTRL);
	return 0;
fail_cleanup:
	dma_free_coherent(&pci_ctx->pdev->dev, q->num_desc_pages * PAGE_SIZE,
			q->ring_virt_addr, q->ring_dma_addr);
	return -1;
}

static int ifc_mcdma_queue_reset(struct ifc_mcdma_queue *q)
{
	ktime_t start, now;
	int val;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	/* assert reset */
	writel(1, q->qcsr + QDMA_REGS_2_Q_RESET);
	start = ktime_get();

	/* wait for reset to deassert */
	while (1) {
		val = readl(q->qcsr + QDMA_REGS_2_Q_RESET);
		if (!val)
			return 0;
		now = ktime_get();
		if (ktime_to_ms(ktime_sub(now, start)) > 5)
			break;
	}

	return -1;
}

static int ifc_eventfd_init(struct eventfd_ctx **ectx, int fd)
{
	int err;

	if (*ectx != NULL)
		eventfd_ctx_put(*ectx);

	*ectx = eventfd_ctx_fdget(fd);
	if (IS_ERR(*ectx)) {
		err = -PTR_ERR(*ectx);
		pr_err("Failed to initialize eventfd: err:%d efd:%u\n",
		       err, fd);
		return err;
	}

	return 0;
}

static void ifc_eventfd_free(struct eventfd_ctx *ectx)
{
	if (ectx != NULL)
		eventfd_ctx_put(ectx);
	ectx = NULL;
}

static int
ifc_mcdma_queue_init(struct ifc_mcdma_queue *q, int dir, void *qcsr)
{
	int ret;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	q->qcsr = qcsr;
	q->num_desc_pages = IFC_MCDMA_DESC_PAGES;
	q->dir = dir;

	ret = ifc_mcdma_queue_reset(q);
	if (ret < 0)
		return ret;
	ret = ifc_mcdma_descq_init(q);
	if (ret < 0)
		return ret;
	return 0;
}

static void ifc_mcdma_queue_free(struct ifc_mcdma_queue *q)
{
	struct ifc_pci_dev *pci_ctx;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return;
	}

	/* deallocate the ring buffer */
	pci_ctx = &q->chnl->dev_ctx->pdev_ctx->pctx;
	dma_free_coherent(&pci_ctx->pdev->dev, q->num_desc_pages * PAGE_SIZE,
			  q->ring_virt_addr, q->ring_dma_addr);
	kfree(q->ctx);

	/* disable descriptor fetching */
	writel(0U, q->qcsr + QDMA_REGS_2_Q_CTRL);

	/* free irq handler */
	if (q->ire)
		ifc_irq_handler_free(&q->chnl->dev_ctx->pdev_ctx->msix_info, q);

	/* free eventfds */
	ifc_eventfd_free(q->ectx);

	pr_debug("Chnl%d: %s Queue successfully freed\n", q->chnl->channel_id,
		 q->dir == IFC_MCDMA_DIR_TX ? "Tx" : "Rx");
}

static int ifc_mcdma_chnl_avail(struct ifc_chr_dev_ctx *dctx, void **csr)
{
	bool *channel_bitmap = dctx->channel_bitmap;
	int i;

	if (unlikely(dctx == NULL)) {
		pr_err("Invalid Device Context\n");
		return -EFAULT;
	}

	*csr = dctx->qcsr;
	for (i = 0; i < IFC_MCDMA_CHANNEL_MAX; i++) {
		if (channel_bitmap[i] == true)
			continue;
		channel_bitmap[i] = true;
		return i;
	}

	return -1;
}

static void ifc_mcdma_chnl_put(struct ifc_chr_dev_ctx *dctx, int chnl_id)
{
	dctx->channel_bitmap[chnl_id] = false;
	dctx->channel_context[chnl_id] = NULL;
}

#ifdef IFC_AVST_MULTI_PORT
static void ifc_mcdma_avoid_hol(void *qcsr)
{
	/* avoid hol */
	writel(0x00, qcsr + QDMA_REGS_2_RSVD_9);
}
#endif

static int
ifc_mcdma_chnl_init(struct ifc_mcdma_channel *chnl_ctx, void *dev_qcsr)
{
	int ret;

	if (unlikely(chnl_ctx == NULL || dev_qcsr == NULL)) {
		pr_err("Invalid Channel Context\n");
		return -EFAULT;
	}

	/* init the queue contexts */
	chnl_ctx->tx.chnl = chnl_ctx;
	chnl_ctx->rx.chnl = chnl_ctx;

	ret = ifc_mcdma_queue_init(&chnl_ctx->rx, IFC_MCDMA_DIR_RX,
				   dev_qcsr + chnl_ctx->channel_id * 256);
	if (ret < 0) {
		pr_err("Chnl%d: Failed to initialize Rx context\n", chnl_ctx->channel_id);
		return ret;
	}
	ret = ifc_mcdma_queue_init(&chnl_ctx->tx,
				   IFC_MCDMA_DIR_TX,
				   dev_qcsr + (1UL << 19) +
				   chnl_ctx->channel_id * 256);
	if (ret < 0) {
		pr_err("Chnl%d: Failed to initialize Tx context\n", chnl_ctx->channel_id);
		return ret;
	}

#ifdef IFC_AVST_MULTI_PORT
	ifc_mcdma_avoid_hol(dev_qcsr);
#endif

	pr_debug("Chnl%d: Queue Initialization successful\n",
		 chnl_ctx->channel_id);

	INIT_LIST_HEAD(&chnl_ctx->mem_list);
	mutex_init(&chnl_ctx->mem_lock);
	mutex_init(&chnl_ctx->poll_lock);

	return 0;
}

static void ifc_mcdma_chnl_free(struct ifc_mcdma_channel *chnl_ctx)
{
	struct ifc_pci_dev *pci_ctx;

	pci_ctx = &chnl_ctx->dev_ctx->pdev_ctx->pctx;
	if (unlikely(pci_ctx == NULL)) {
		pr_err("Invalid PCI Context\n");
		return;
	}

#ifdef DEBUG
	/* print qstats */
	//ifc_mcdma_print_qstats(&chnl_ctx->rx);
	//ifc_mcdma_print_qstats(&chnl_ctx->tx);
#endif

	/* free the queue contexts */
	ifc_mcdma_queue_free(&chnl_ctx->rx);
	ifc_mcdma_queue_free(&chnl_ctx->tx);

#ifdef IFC_MCDMA_KMEM
	/* free dma memory */
	ifc_dma_mem_free(&pci_ctx->pdev->dev, chnl_ctx->dev_ctx->dmem_pool,
			 &chnl_ctx->mem_list);
#endif
	mutex_lock(&chnl_ctx->dev_ctx->chnl_lock);
	ifc_mcdma_chnl_put(chnl_ctx->dev_ctx, chnl_ctx->channel_id);
	mutex_unlock(&chnl_ctx->dev_ctx->chnl_lock);
	kfree(chnl_ctx);
}

int ifc_mcdma_ip_reset(struct ifc_chr_dev_ctx *dctx)
{
	void *base;

	if (unlikely(dctx == NULL || dctx->qcsr == NULL))
		return -EFAULT;

	base = dctx->qcsr;
	mdelay(500UL);

	/* IP reset */
	writel(0x01, base + QDMA_REGS_2_SOFT_RESET);

	/* giving time for the FPGA reset to complete */
	mdelay(500UL);

	pr_debug("Successfully reset IP\n");
	return 0;
}

static int ifc_device_open(struct inode *ifc_inode, struct file *ifc_filp)
{
	struct ifc_mcdma_channel **chnl_ctx_list;
	struct ifc_mcdma_channel *chnl_ctx;
	struct ifc_chr_dev_ctx *dctx;
	void *dev_qcsr;
	int chnl_id;
	int ret = 0;

	dctx = container_of(ifc_inode->i_cdev, struct ifc_chr_dev_ctx,
			    ifc_cdev);
	if (unlikely(dctx == NULL)) {
		pr_err("Failed to retrieve device context\n");
		return -EFAULT;
	}

	pr_debug("Device opened: %s\n", ifc_filp->f_path.dentry->d_iname);
	chnl_ctx_list = dctx->channel_context;

	/* search for a channel */
	mutex_lock(&dctx->chnl_lock);
	chnl_id = ifc_mcdma_chnl_avail(dctx, &dev_qcsr);
	if (chnl_id < 0) {
		pr_err("No available Channel for use\n");
		mutex_unlock(&dctx->chnl_lock);
		return -1;
	}

	pr_debug("Allocating channel %d\n", chnl_id);

	/* allocate memory for the channel context */
	chnl_ctx = kzalloc(sizeof(struct ifc_mcdma_channel), GFP_KERNEL);
	if (chnl_ctx == NULL) {
		mutex_unlock(&dctx->chnl_lock);
		return -ENOMEM;
	}

	/* init the context */
	chnl_ctx->channel_id = chnl_id;
	chnl_ctx->dev_ctx = dctx;

#ifndef IFC_MCDMA_BAS_EN
	ret = ifc_mcdma_chnl_init(chnl_ctx, dev_qcsr);
	if (ret < 0) {
		mutex_unlock(&dctx->chnl_lock);
		pr_err("Failed to Initialize Channel Context\n");
		goto fail_cleanup;
	}
#endif

	chnl_ctx_list[chnl_id] = chnl_ctx;
	ifc_filp->private_data = chnl_ctx;
#ifdef IFC_MCDMA_BAS_EN
	chnl_ctx->direction = -1;
#endif
	mutex_unlock(&dctx->chnl_lock);

	pr_debug("Chnl%d: Successfully Opened Char dev file\n",
		 chnl_ctx->channel_id);
	return 0;

fail_cleanup:
	kfree(chnl_ctx);
	return ret;
}

static int ifc_device_release(struct inode *ifc_inode, struct file *ifc_filp)
{
	struct ifc_mcdma_channel *chnl_ctx;

	chnl_ctx = ifc_filp->private_data;
	if (unlikely(chnl_ctx == NULL)) {
		pr_err("Invalid Channel Context\n");
		return -EFAULT;
	}
#if defined(IFC_MCDMA_BAS_EN)
	ifc_release_msi_interrupts(chnl_ctx);
#else
	ifc_mcdma_chnl_free(chnl_ctx);
#endif
	pr_debug("Successfully Closed Char dev file\n");
	return 0;
}

static void ifc_vma_open(struct vm_area_struct *vma)
{
/*
 *	printk(KERN_DEBUG "Opening VMA region with start addr: %lu\n",
 *		vma->vm_start);
 */
}

static void ifc_vma_close(struct vm_area_struct *vma)
{
	struct ifc_dma_mem_info *mem_info;

	if (unlikely(vma == NULL)) {
		pr_err("Invalid VMA Context\n");
		return;
	}
	mem_info = (struct ifc_dma_mem_info *)vma->vm_private_data;
	dma_pool_free(mem_info->dmem_pool, mem_info->mem_virt_addr,
		      mem_info->mem_dma_addr);

	mutex_lock(mem_info->mem_lock);
	list_del(&mem_info->list);
	mutex_unlock(mem_info->mem_lock);

	kfree(mem_info);
}

static struct vm_operations_struct vops = {
	.open = ifc_vma_open,
	.close = ifc_vma_close
};


static int ifc_device_mmap(struct file *filp, struct vm_area_struct *vma)
{
	struct ifc_mcdma_channel *chnl = (struct ifc_mcdma_channel *)
					 (filp->private_data);
	struct ifc_dma_mem_info *mem_info;
	struct dma_pool *dmem_pool = NULL;
	size_t dmem_pool_size;
	phys_addr_t phy_addr;
	size_t size;
	int ret;

	if (unlikely(chnl == NULL)) {
		pr_err("Invalid Channel Context\n");
		return -EFAULT;
	}

	dmem_pool = chnl->dev_ctx->dmem_pool;
	dmem_pool_size = chnl->dev_ctx->dmem_pool_size;

	if (unlikely(dmem_pool == NULL)) {
		pr_err("Invalid Channel Context\n");
		return -EFAULT;
	}

	if (unlikely(vma == NULL)) {
		pr_err("Invalid Application Context\n");
		return -EINVAL;
	}

	mem_info = kzalloc(sizeof(struct ifc_dma_mem_info), GFP_KERNEL);
	if (mem_info == NULL)
		return -ENOMEM;

	size = vma->vm_end - vma->vm_start;
	if (size > dmem_pool_size) {
		kfree(mem_info);
		return -EINVAL;
	}

	ret = ifc_dma_mem_alloc(dmem_pool, dmem_pool_size, &mem_info->mem_virt_addr,
				&mem_info->mem_dma_addr);
	if (ret < 0) {
		kfree(mem_info);
		return ret;
	}
	phy_addr = virt_to_phys(mem_info->mem_virt_addr);
	ret = remap_pfn_range(vma, vma->vm_start, (phy_addr >> PAGE_SHIFT) +
			      vma->vm_pgoff, size, vma->vm_page_prot);
	if (ret != 0) {
		pr_err("mmap handler failed\n");
		kfree(mem_info);
		return -EINVAL;
	}

	vma->vm_ops = &vops;
	vma->vm_private_data = mem_info;

	mem_info->user_virt_addr = vma->vm_start;
	mem_info->dmem_pool = dmem_pool;
	INIT_LIST_HEAD(&mem_info->list);
	mem_info->mem_lock = &chnl->mem_lock;

	mutex_lock(&chnl->mem_lock);
	list_add(&mem_info->list, &chnl->mem_list);
	ifc_vma_open(vma);
	mutex_unlock(&chnl->mem_lock);

	return 0;
}

static int ifc_descq_processed_count(struct ifc_mcdma_queue *q, int *cur_head)
{
	int head;
	int prev_head;

	if (q->wbe || q->ire)
		head = q->consumed_head;
	else
		head = readl(q->qcsr + QDMA_REGS_2_Q_COMPLETED_POINTER);

	q->prev_con_head = q->cur_con_head;
	q->cur_con_head = head;
	prev_head = q->prev_con_head;

	if (cur_head)
		*cur_head = head % q->qlen;

	return (head >= prev_head) ?
		(head - prev_head) : (IFC_NUM_DESC_INDEXES - prev_head + head);
}

static int ifc_descq_empty_slots(struct ifc_mcdma_queue *q)
{
	int max_free_slots = (IFC_MCDMA_DESC_PAGES * 127);
	int head = q->head;
	int tail = q->tail;
	int desc_per_page;
	int free_slots;
	int links;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	if (q->head_didx == q->tail_didx)
		return max_free_slots;

	if ((q->head_didx != q->tail_didx) && (q->head == q->tail))
		return 0;

	desc_per_page = PAGE_SIZE / sizeof(struct ifc_mcdma_desc);

	if (head < tail)
		free_slots = q->qlen - tail + head;
	else
		free_slots = head - tail;

	links = (tail / desc_per_page - head / desc_per_page);

	if (head >= tail)
		free_slots += links;
	else
		free_slots = free_slots - (q->num_desc_pages - links);

	return free_slots;
}

#if defined(HW_FETCH_HEAD_TRACKER)
static int hw_fetch_ptr_trckr(struct ifc_mcdma_queue *q)
{
	uint32_t qhead;

	if (q->tid_updates) {
		qhead = readl(q->qcsr + QDMA_REGS_2_Q_HEAD_POINTER);
		if (q->processed_tail != qhead)
			return 0;
		q->tid_updates = 0;
	}
	return 1;
}

#elif defined(HW_TID_DEPTH_TRACKER)
#ifdef IFC_AVST_MULTI_PORT
static int fifo_len_multiport(struct ifc_mcdma_queue *q)
{
	uint32_t off;

	/*
	 * check for fifo_len. In case if it reaches 0, read latest
	 * values from DMQ CSR
	 */
	if (q->fifo_len == 0) {
		if (q->dir)
			off = QDMA_REGS_2_P0_H2_D_TPTR_AVL +
				(q->chnl->channel_id * 4);
		else
			off = QDMA_REGS_2_P0_D2_H_TPTR_AVL +
				(q->chnl->channel_id * 4);

		q->fifo_len = readl(q->chnl->dev_ctx->qcsr + off);
	}
	return q->fifo_len;
}
#else	/* Single port */
static int fifo_len_singleport(struct ifc_mcdma_queue *q)
{
	volatile uint32_t fifo_len;
	uint32_t space = 384U;
	uint32_t off;

#ifndef IFC_QDMA_INTF_AVST
	mutex_lock(&q->chnl->dev_ctx->tid_lock);
#endif
        if (q->dir)
                off = QDMA_REGS_2_P0_H2_D_TPTR_AVL;
        else
                off = QDMA_REGS_2_P0_D2_H_TPTR_AVL;

        fifo_len = readl((char *)q->chnl->dev_ctx->qcsr + off);

#ifndef IFC_QDMA_INTF_AVST
	mutex_unlock(&q->chnl->dev_ctx->tid_lock);
	q->fifo_len = fifo_len;
#else
        if (fifo_len < space)
                q->fifo_len = 0;
	else
		q->fifo_len = 1;
#endif
	return q->fifo_len;
}
#endif	/* IFC_AVST_MULTI_PORT */

static int hw_tid_fifo_trckr(struct ifc_mcdma_queue *q)
{
#ifdef IFC_AVST_MULTI_PORT
	return fifo_len_multiport(q);
#else
	return fifo_len_singleport(q);
#endif
}
#endif	/* HW_FETCH_HEAD_TRACKER */

static int get_fifo_len(struct ifc_mcdma_queue *q)
{
#if defined(HW_FETCH_HEAD_TRACKER)
	return hw_fetch_ptr_trckr(q);
#elif defined(HW_TID_DEPTH_TRACKER)
	return hw_tid_fifo_trckr(q);
#else
	return 1;
#endif
}

/* AVMM address computation */
static inline uint64_t
avmm_addr_manager(struct ifc_mcdma_queue *q, uint64_t *addr,
		  unsigned long payload)
{
	uint64_t start_addr;

	//*addr = (*addr + payload < IFC_AVMM_BUF_LIMIT) ? *addr : q->base;
	start_addr = *addr;
	*addr = (*addr + payload);

	return start_addr;
}

static int
ifc_mcdma_desc_prep(struct ifc_mcdma_queue *q, uint64_t mem_phy_addr,
			size_t pyld_cnt)
{
	struct ifc_mcdma_desc *d;
	uint32_t tail;
	uint64_t addr = 0ULL;

	if (unlikely(q == NULL || mem_phy_addr == 0UL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}
#ifdef DEBUG
	pr_debug("Chnl%d,%s:desc_prep in\n",
		 q->chnl->channel_id, (q->dir == IFC_MCDMA_DIR_TX? "TX": "RX"));
#endif

	/* check if descq is full */
	if (ifc_descq_empty_slots(q) <= 0)
		return -1;

	/* check if fifo is full */
	if (q->fifo_len <= 0 && get_fifo_len(q) == 0)
		return -1;

	tail = q->tail;
	d = (struct ifc_mcdma_desc *)
	    ((uint64_t)q->ring_virt_addr + sizeof(*d) * tail);
	if (d->link) {  /* skip the link */
		tail = (tail + 1) % q->qlen;
		q->didx = (q->didx + 1) % IFC_NUM_DESC_INDEXES;
		d = (struct ifc_mcdma_desc *)
		    ((uint64_t)q->ring_virt_addr + sizeof(*d) * tail);
	}

	memset(d, 0, sizeof(struct ifc_mcdma_desc));

	if (pyld_cnt > IFC_MCDMA_BUF_LIMIT)
		return -EINVAL;

#ifndef IFC_QDMA_INTF_AVST
	/* set the address */
	addr = avmm_addr_manager(q, &q->addr, pyld_cnt);
#endif

	switch (q->dir) {
	case IFC_MCDMA_DIR_RX:
		d->src = addr;
		d->dest = mem_phy_addr;
		break;
	case IFC_MCDMA_DIR_TX:
		d->src = mem_phy_addr;
		d->dest = addr;
		break;
	default:
		break;
	}

	d->len = pyld_cnt;
	q->ctx[tail].ctx_data = (void *)mem_phy_addr;
	q->tail = (tail + 1) % q->qlen;
	q->didx = (q->didx + 1) % IFC_NUM_DESC_INDEXES;
	q->prep_counter++;
	d->didx = q->didx;
	d->wb_en = 0;
	d->msix_en = 0;
	d->pad_len = 0;

	q->tail_didx = q->didx;

#ifdef IFC_QDMA_INTF_AVST
	if (q->dir == IFC_MCDMA_DIR_TX) {
		if (q->prep_counter == 1)
			d->sof = 1;
		if (q->file_size && q->prep_counter == q->file_size) {
			d->eof = 1;
			q->prep_counter = 0;
		}
	}
#endif
#ifdef DEBUG
	pr_debug("Chnl%d,%s:desc_prep out; src=%llx, dest=%llx\n",
		 q->chnl->channel_id, (q->dir == IFC_MCDMA_DIR_TX? "TX": "RX"),
		 d->src, d->dest);
#endif
	return 0;
}

#ifdef IFC_MCDMA_KMEM
static int
ifc_mcdma_desc_prep(struct ifc_mcdma_queue *q,
		    struct ifc_dma_mem_info *mem_info, size_t len)
{
	struct request_struct *req;
	struct ifc_mcdma_desc *d;
	void *dma_buf_virt;
	uint64_t dma_buf;
	uint32_t tail;

	if (unlikely(q == NULL || mem_info == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	/* check if descq is full */
	if (ifc_descq_empty_slots(q) <= 0)
		return -1;

	/* check if fifo is full */
	if (q->fifo_len <= 0 && get_fifo_len(q) == 0)
		return -1;

	tail = q->tail;
	d = (struct ifc_mcdma_desc *)
	    ((uint64_t)q->ring_virt_addr + sizeof(*d) * tail);
	if (d->link) {  /* skip the link */
		tail = (tail + 1) % q->qlen;
		q->didx = ((q->didx + 1) % IFC_NUM_DESC_INDEXES);
		d = (struct ifc_mcdma_desc *)
		    ((uint64_t)q->ring_virt_addr + sizeof(*d) * tail);
	}

	memset(d, 0, sizeof(struct ifc_mcdma_desc));
	req = (struct request_struct *)mem_info->mem_virt_addr;

#ifndef IFC_QDMA_INTF_AVST
	if (q->dir == IFC_MCDMA_DIR_RX && req->src > IFC_AVMM_BUF_LIMIT)
		return -EINVAL;
	else if (q->dir == IFC_MCDMA_DIR_TX && req->dest > IFC_AVMM_BUF_LIMIT)
		return -EINVAL;
#endif
	if (req->pyld_cnt > (1 << 20))
		return -EINVAL;
	if (req->flags > 3U)
		return -EINVAL;

	dma_buf_virt = (void *)((uint64_t)(mem_info->mem_virt_addr) +
			sizeof(struct request_struct));
	dma_buf = virt_to_phys(dma_buf_virt);


	switch (q->dir) {
	case IFC_MCDMA_DIR_RX:
		d->src = req->src;
		d->dest = (uint64_t)dma_buf;
		break;
	case IFC_MCDMA_DIR_TX:
		d->src = (uint64_t)dma_buf;
		d->dest = req->dest;
		break;
	default:
		break;
	}

#ifdef IFC_QDMA_INTF_AVST
	 /* For Rx, we don't know the start and end of the file */
	if (q->dir == IFC_MCDMA_DIR_TX) {
		if  ((d->len % 64 != 0) && d->eof == 0) {
			pr_err("Invalid length for Non-EOF descriptor\n");
			return -1;
		}
	}
#endif
	d->len = len;
	q->ctx[tail].ctx_data = (void *)mem_info;
	q->tail = (tail + 1) % q->qlen;
	q->didx = ((q->didx + 1) % IFC_NUM_DESC_INDEXES);
	d->didx = q->didx;
	d->wb_en = 0;
	d->msix_en = 0;
	d->pad_len = 0;

	return 0;
}
#endif /* IFC_MCDMA_KMEM */

static int ifc_mcdma_desc_submit(struct ifc_mcdma_queue *q)
{
	struct ifc_mcdma_desc *d;
	int index;
	int count = 0;
	uint32_t qhead;

	if (unlikely(q == NULL)) {
		pr_err("Invalid Queue Context\n");
		return -EFAULT;
	}

	/* update completion mechanism */
	index = q->tail - 1;
	d = (struct ifc_mcdma_desc *)
	    ((uint64_t)q->ring_virt_addr + sizeof(*d) * index);

	if (q->wbe || q->ire)
		d->wb_en = 1;
	d->msix_en = q->ire;
#ifdef IFC_QDMA_INTF_AVST
	if (!q->file_size) {
		d->eof = 1;
		q->prep_counter = 0;
	}
#endif

	if (q->tail == q->processed_tail && q->processed_tail == q->qlen - 1) {
		writel(q->qlen, q->qcsr + QDMA_REGS_2_Q_TAIL_POINTER);
		while (count < IFC_CNT_HEAD_MOVE) {
			qhead = readl(q->qcsr + QDMA_REGS_2_Q_HEAD_POINTER);
			if ((qhead % 128) == 0)
				break;
			count++;
		}
	}
	/* update tail pointer */
	writel(q->tail, q->qcsr + QDMA_REGS_2_Q_TAIL_POINTER);

	/* update last submitted tail */
	q->processed_tail = q->tail;

	q->tid_updates++;
	q->fifo_len--;
	return 0;
}

static int
ifc_ctrl_msg_handler(struct ifc_mcdma_channel *chnl, const char __user *buf)
{
	int max_free_slots = (IFC_MCDMA_DESC_PAGES * 127);
	struct ctrl_message cmsg;
	int *efd;
	int ret;

	ret = copy_from_user(&cmsg, buf, sizeof(struct ctrl_message));
	if (ret != 0) {
		pr_err("Failed to copy ctrl message\n");
		return -1;
	}

	if (cmsg.fefd_tx) {
		efd = &cmsg.efd_tx;
		if (IS_ERR(efd))
			return -1;

		ret = ifc_eventfd_init(&chnl->tx.ectx, cmsg.efd_tx);
		if (ret != 0)
			return ret;
#ifdef IFC_MCDMA_BAS_EN
		chnl->direction = 1;
		chnl->rx.irq_index = -1;
		chnl->rx.dir = 1;
		ret = ifc_msi_irq_handler_alloc(chnl, &(chnl->tx),
			&(chnl->dev_ctx->pdev_ctx->msi_info), 1);
		if (ret != 0){
			pr_err("Failed to register IRQ\n");
			return ret;
		}
#endif
	}

	if (cmsg.fefd_rx) {
		efd = &cmsg.efd_rx;
		if (IS_ERR(efd))
			return -1;
		ret = ifc_eventfd_init(&chnl->rx.ectx, cmsg.efd_rx);
		if (ret != 0)
			return ret;
#ifdef IFC_MCDMA_BAS_EN
		chnl->direction = 0;
		chnl->rx.irq_index = -1;
		chnl->rx.dir = 0;
		ret = ifc_msi_irq_handler_alloc(chnl, &(chnl->rx),
			&(chnl->dev_ctx->pdev_ctx->msi_info), 0);
		if (ret != 0){
			pr_err("Failed to register IRQ\n");
			return ret;
		}
#endif
	}

	if (cmsg.tx_fsize) {
		if (cmsg.tx_fsize < 0 || cmsg.tx_fsize > max_free_slots)
			return -1;
		chnl->tx.file_size = cmsg.tx_fsize;
	}

	if (cmsg.rx_fsize) {
		if (cmsg.rx_fsize < 0 || cmsg.rx_fsize > max_free_slots)
			return -1;
		chnl->rx.file_size = cmsg.rx_fsize;
	}

	if (cmsg.tx_payload) {
		if (cmsg.tx_payload < 0 || cmsg.tx_payload > IFC_MCDMA_BUF_LIMIT)
			return -1;
		chnl->tx.payload = cmsg.tx_payload;
	}

	if (cmsg.rx_payload) {
		if (cmsg.rx_payload < 0 || cmsg.rx_payload > IFC_MCDMA_BUF_LIMIT)
			return -1;
		chnl->rx.payload = cmsg.rx_payload;
		#ifdef IFC_QDMA_INTF_AVST
		/* configure payload */
		writel(ifc_check_payload(chnl->rx.payload),((char*)(chnl->rx.qcsr) + QDMA_REGS_2_Q_DEBUG_STATUS_4));
		#endif

	}
	pr_debug("Chnl%d: Successfully set up eventfd context\n",
		 chnl->channel_id);
	return 1;
}

#ifdef IFC_MCDMA_KMEM
static struct vm_area_struct *ifc_find_vma(struct mm_struct *mm, unsigned long addr)
{
	struct vm_area_struct *vma = NULL;
	struct rb_node *rb_node;

	rb_node = mm->mm_rb.rb_node;

	while (rb_node) {
		struct vm_area_struct *vma_tmp;

		vma_tmp = rb_entry(rb_node,
				struct vm_area_struct, vm_rb);

		if (vma_tmp->vm_end > addr) {
			vma = vma_tmp;
			if (vma_tmp->vm_start <= addr)
				break;
			rb_node = rb_node->rb_left;
		} else
			rb_node = rb_node->rb_right;
	}
	return vma;
}
#endif /* IFC_MCDMA_KMEM */

/*
static struct page *addr_to_page(struct mm_struct *mm, unsigned long addr)
{
	struct page *pg = NULL;
	pgd_t *pgd;
	pud_t *pud;
	pmd_t *pmd;
	pte_t *ptep, pte;

	pgd = pgd_offset(mm, addr);
	if (pgd_none(*pgd) || pgd_bad(*pgd))
		return NULL;
	printk(KERN_INFO "PGD entry found\n");

	pud = pud_offset(pgd, addr);
	if (pud_none(*pud) || pud_bad(*pud))
		return NULL;
	printk(KERN_INFO "PUD entry found\n");

	pmd = pmd_offset(pud, addr);
	if (pmd_none(*pmd) || pmd_bad(*pmd))
		return NULL;
	printk(KERN_INFO "PMD entry found\n");

	ptep = pte_offset_kernel(pmd, addr);
	if (pte_none(*ptep) || !pte_present(*ptep))
		printk(KERN_INFO "Page Table entry not found\n");

	printk(KERN_INFO "Page Table entry: %lu\n", pte_val(*ptep));

	pte = *ptep;
	pg = pte_page(pte);

	return pg;
}
*/

#if 0
static int
ifc_kmap_buffer(struct vm_area_struct *vma, struct ifc_dma_mem_info **mem_info)
{
	struct ifc_dma_mem_info *minfo;
	struct page **pages;
	void *virt_addr;
	int rcd_pages;
	int nr_pages;
	size_t size;
	int i;

	if (vma == NULL)
		return -1;

	/* VMA page count */
	size = vma->vm_end - vma->vm_start;
	nr_pages = size / PAGE_SIZE;
	if (nr_pages <= 0)
		return -1;

	/* create entry for the memory area */
	minfo = kzalloc(sizeof(struct ifc_dma_mem_info), GFP_KERNEL);
	if (minfo == NULL)
		return -ENOMEM;

	/* get the VMA pages */
	pages = kcalloc(nr_pages, sizeof(struct page *), GFP_KERNEL);
	if (pages == NULL) {
		kfree(minfo);
		return -ENOMEM;
	}
	rcd_pages = get_user_pages_fast(vma->vm_start, nr_pages, 1, pages);
	if (rcd_pages != nr_pages)
		goto err;

	/* map the pages to kernel address space */
	for (i = 0; i < nr_pages; i++) {
		virt_addr = kmap(pages[i]);
		if (i == 0)
			minfo->mem_virt_addr = virt_addr;
	}

	if (virt_addr == NULL) {
		printk(KERN_INFO "%s:%d: Mapping failed\n", __func__, __LINE__);
		goto err;
	}

	minfo->mem_virt_addr = virt_addr;
	minfo->user_virt_addr = vma->vm_start;
	minfo->pages = pages;
	minfo->nr_pages = rcd_pages;
	minfo->size = size;
	*mem_info = minfo;
	return 0;
err:
#if 0
	for (i = 0; i < rcd_pages; i++)
		put_page(pages[i]);
#endif

	kfree(pages);
	kfree(minfo);
	return -1;
}
#endif

static ssize_t
ifc_device_read_write(struct file *filp, const char __user *buf,
		      size_t count, int dir, bool do_submit)
{
	struct ifc_mcdma_channel *chnl_ctx = (struct ifc_mcdma_channel *)
					     (filp->private_data);
	struct ifc_mcdma_queue *q;
#ifdef IFC_MCDMA_KMEM
	struct ifc_dma_mem_info *vma_info;
#endif
	uint64_t mem_phy_addr;
	int ret;

	if (chnl_ctx == NULL) {
		pr_err("Invalid Parameters Received\n");
		return -1;
	}
#ifdef DEBUG
	pr_debug("Chnl%d,%s:read_write in, do_submit=%s\n",
		 chnl_ctx->channel_id, (dir == IFC_MCDMA_DIR_TX? "TX": "RX"), (do_submit == true? "True": "False"));
#endif

	if (buf == NULL || count > IFC_MCDMA_BUF_LIMIT) {
		pr_err("Invalid Parameters Received\n");
		return -1;
	}

	if (dir != IFC_MCDMA_DIR_TX && dir != IFC_MCDMA_DIR_RX) {
		pr_err("Invalid Parameters Received\n");
		return -1;
	}

	/* control messge */
	if (count == 0) {
		ret = ifc_ctrl_msg_handler(chnl_ctx, buf);
		if (ret < 0) {
			pr_err("Failed to process Control message\n");
			return -1;
		}
		return ret;
	}

#ifdef IFC_MCDMA_KMEM
	vma = find_vma(current->mm, (unsigned long)buf);
	if (vma == NULL ||
			(unsigned long)buf < vma->vm_start ||
			(unsigned long)buf >= vma->vm_end) {
		vma = ifc_find_vma(current->mm, (unsigned long)buf);
		if (vma == NULL)
			return -1;

		if ((unsigned long)buf < vma->vm_start || (unsigned long)buf >= vma->vm_end)
			return -1;
	}
	vma_info = (struct ifc_dma_mem_info *)(vma->vm_private_data);
	if (vma_info == NULL)
		return -1;

	if (dir == IFC_MCDMA_DIR_TX)
		q = &chnl_ctx->tx;
	else
		q = &chnl_ctx->rx;

	/* prep the request */
	ret = ifc_mcdma_desc_prep(q, vma_info, count);
	if (ret < 0)
		return ret;
#else
	mem_phy_addr = (uint64_t)buf;
	if (dir == IFC_MCDMA_DIR_TX)
		q = &chnl_ctx->tx;
	else
		q = &chnl_ctx->rx;

	/* prep the request */
	ret = ifc_mcdma_desc_prep(q, mem_phy_addr, count);
	if (ret < 0)
		return -1;
#endif

	/* submit it */
	if (do_submit) {
		ret = ifc_mcdma_desc_submit(q);
		if (ret < 0)
			return -1;
	}

	return count;
}

static ssize_t
ifc_device_read(struct file *filp, char __user *buf, size_t count,
		loff_t *offp)
{
	return ifc_device_read_write(filp, buf, count, IFC_MCDMA_DIR_RX, true);
}

static ssize_t
ifc_device_write(struct file *filp, const char __user *buf, size_t count,
		 loff_t *offp)
{
	return ifc_device_read_write(filp, buf, count, IFC_MCDMA_DIR_TX, true);
}

static int ifc_refresh_queue_state(struct ifc_mcdma_queue *q)
{
	q->qstate.head = q->head;
	q->qstate.tail = q->tail;
	q->qstate.didx = q->didx;
	q->qstate.head_didx = q->head_didx;
	q->qstate.tail_didx = q->tail_didx;
	q->qstate.prep_counter = q->prep_counter;
	q->qstate.file_size = q->file_size;
	q->qstate.payload = q->payload;
#ifndef IFC_QDMA_INTF_AVST
	q->qstate.base = q->base;
	q->qstate.addr = q->addr;
#endif
	return 0;
}

static int ifc_restore_queue_state(struct ifc_mcdma_queue *q)
{
	q->head = q->qstate.head;
	q->tail = q->qstate.tail;
	q->didx = q->qstate.didx;
	q->head_didx = q->qstate.head_didx;
	q->tail_didx = q->qstate.tail_didx;
	q->prep_counter = q->qstate.prep_counter;
	q->file_size = q->qstate.file_size;
	q->payload = q->qstate.payload;
#ifndef IFC_QDMA_INTF_AVST
	q->base = q->qstate.base;
	q->addr = q->qstate.addr;
#endif
	return 0;
}

static ssize_t
ifc_read_write_iter(struct file *filp, const struct iovec *iov,
		    unsigned long nr_segs, int dir)
{
	struct ifc_mcdma_channel *chnl =
	(struct ifc_mcdma_channel *)filp->private_data;
	struct ifc_mcdma_queue *q;
	size_t count = 0;
	bool do_submit;
	int empty_slots;
	int ret;
	int i;

#ifdef DEBUG
	if (nr_segs != 0 && iov != NULL)
		pr_debug("nr_segs=%ld, iov_base=%llx, iov_len=%lx\n", nr_segs, (uint64_t)iov[0].iov_base, iov[0].iov_len);
#endif
	if (chnl == NULL) {
		pr_err("Invalid Channel Context\n");
		return -EINVAL;
	}

	if (dir == IFC_MCDMA_DIR_TX)
		q = &chnl->tx;
	else
		q = &chnl->rx;

	if (nr_segs == 1 && iov[0].iov_len == (IFC_MCDMA_BUF_LIMIT << 4)) {
		q->addr = (uint64_t)iov[0].iov_base;
		return 0;
	}

	/* check if descq have enough empty slots */
	empty_slots = ifc_descq_empty_slots(q);
	if (empty_slots < nr_segs) {
		ifc_queue_poll(q);
		if (ifc_descq_empty_slots(q) < nr_segs)
			return -1;
	}

	/* check if fifo is full */
	if (q->fifo_len <= 0 && get_fifo_len(q) == 0)
		return -1;

	ifc_refresh_queue_state(q);

	for (i = 0; i < nr_segs; i++) {
		do_submit = false;
		if (i + 1 == nr_segs)
			do_submit = true;
		ret = ifc_device_read_write(filp, iov[i].iov_base,
					    iov[i].iov_len, dir, do_submit);
		if (ret <= 0) {
			ifc_restore_queue_state(q);
			return -1;
		}

		count += iov[i].iov_len;
	}
	return count;
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 1, 0)
static ssize_t
ifc_device_read_iter(struct kiocb *iocb, const struct iovec *iov,
		     unsigned long nr_segs, loff_t pos)
{
#else
static ssize_t
ifc_device_read_iter(struct kiocb *iocb, struct iov_iter *iv)
{
	const struct iovec *iov = iv->iov;
	unsigned long nr_segs = iv->nr_segs;
#endif
	struct file *filp = (struct file *)iocb->ki_filp;

	return ifc_read_write_iter(filp, iov, nr_segs, IFC_MCDMA_DIR_RX);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 1, 0)
static ssize_t
ifc_device_write_iter(struct kiocb *iocb, const struct iovec *iov,
		      unsigned long nr_segs, loff_t pos)
{
#else
static ssize_t
ifc_device_write_iter(struct kiocb *iocb, struct iov_iter *iv)
{
	const struct iovec *iov = iv->iov;
	unsigned long nr_segs = iv->nr_segs;
#endif
	struct file *filp = (struct file *)iocb->ki_filp;

	return ifc_read_write_iter(filp, iov, nr_segs, IFC_MCDMA_DIR_TX);
}

static unsigned int
ifc_device_poll(struct file *filp, struct poll_table_struct *wait)
{
	unsigned int ret = 0U;
	struct ifc_mcdma_channel *chnl_ctx = (struct ifc_mcdma_channel *)
					     (filp->private_data);
	struct ifc_mcdma_queue *tx = &chnl_ctx->tx;
	struct ifc_mcdma_queue *rx = &chnl_ctx->rx;
	int nr;
	unsigned int req_events;

	if (chnl_ctx == NULL) {
		pr_err("Invalid Channel Context\n");
		return -EINVAL;
	}

	req_events = poll_requested_events(wait);

	if (req_events & (POLLOUT | POLLWRNORM)) {
		nr = ifc_queue_poll(tx);
		if (tx->head != tx->tail)
			ret |= POLLOUT;
		else if (tx->head_didx == tx->tail_didx) {
			ret |= POLLWRNORM;
			ret |= POLLOUT;
		}
		if (tx->ectx && nr > 0)
			eventfd_signal(tx->ectx, nr);
	}

	if (req_events & (POLLIN | POLLRDNORM)) {
		nr = ifc_queue_poll(rx);
		if (rx->head != rx->tail)
			ret |= POLLIN;
		else if (rx->head_didx == rx->tail_didx) {
			ret |= POLLRDNORM;
			ret |= POLLIN;
		}
		if (rx->ectx && nr > 0)
			eventfd_signal(rx->ectx, nr);
	}
	return ret;
}

static struct file_operations ifc_fops = {
	.owner = THIS_MODULE,
	.open = ifc_device_open,
	.release = ifc_device_release,
	.mmap = ifc_device_mmap,
	.read = ifc_device_read,
	.write = ifc_device_write,
#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 1, 0)
	.aio_read = ifc_device_read_iter,
	.aio_write = ifc_device_write_iter,
#else
	.read_iter = ifc_device_read_iter,
	.write_iter = ifc_device_write_iter,
#endif
	.poll = ifc_device_poll
};


int ifc_mcdma_node_create(struct ifc_chr_dev_ctx *dctx)
{
	int ret;
	dev_t dev_num;

	if (unlikely(dctx == NULL)) {
		pr_err("Invalid Device Context\n");
		return -EFAULT;
	}
	dev_num = MKDEV(dctx->major, dctx->minor);
	cdev_init(&dctx->ifc_cdev, &ifc_fops);
	dctx->ifc_cdev.owner = THIS_MODULE;
	dctx->ifc_cdev.ops = &ifc_fops;

	ret = cdev_add(&dctx->ifc_cdev, dev_num, 1);
	if (ret < 0)
		goto fail_cleanup;

	dctx->ifc_device = device_create(gctx.gclass, NULL, dev_num,
					 NULL, "%s", dctx->chr_dev_name);
	if (IS_ERR(dctx->ifc_device)) {
		ret = 0 - PTR_ERR(dctx->ifc_device);
		dctx->ifc_device = NULL;
		goto fail_cleanup;
	}

	pr_debug("Successfully created char device %s\n", dctx->chr_dev_name);
	return 0;

fail_cleanup:
	cdev_del(&dctx->ifc_cdev);
	return ret;
}


void ifc_mcdma_node_destroy(struct ifc_chr_dev_ctx *dctx)
{
	if (unlikely(dctx == NULL)) {
		pr_err("Invalid Device Context\n");
		return;
	}

	if (dctx->ifc_device) {
		device_destroy(gctx.gclass, MKDEV(dctx->major, dctx->minor));
		dctx->ifc_device = NULL;
	}
	cdev_del(&dctx->ifc_cdev);
}

static unsigned int alloc_minor(struct ifc_mcdma_global_ctx *global_ctx)
{
	int i;

	mutex_lock(&global_ctx->global_lock);
	for (i = 0; i < IFC_MCDMA_DEV_MAX; i++) {
		if (global_ctx->minor_bitmap[i] == false) {
			global_ctx->minor_bitmap[i] = true;
			mutex_unlock(&global_ctx->global_lock);
			return i;
		}
	}
	mutex_unlock(&global_ctx->global_lock);
	return IFC_MCDMA_DEV_MAX;
}

static void
dealloc_minor(struct ifc_mcdma_global_ctx *global_ctx, unsigned int minor)
{
	mutex_lock(&global_ctx->global_lock);
	global_ctx->minor_bitmap[minor] = false;
	mutex_unlock(&global_ctx->global_lock);
}

/* will be called from the pcie probe */
int
ifc_device_chr_init(struct ifc_chr_dev_ctx **dctx,
		    struct ifc_mcdma_dev_ctx *dev_ctx)
{
	struct ifc_pci_dev *pctx;
	struct ifc_chr_dev_ctx *cdev_ctx;
	int ret = 0;
#ifdef IFC_MCDMA_KMEM
	size_t size;
	char pool_name[30];
#endif

	if (unlikely(dev_ctx == NULL)) {
		pr_err("Invalid Device Context\n");
		return -EFAULT;
	}

	pctx = &dev_ctx->pctx;

	cdev_ctx = kzalloc(sizeof(struct ifc_chr_dev_ctx), GFP_KERNEL);
	if (cdev_ctx == NULL)
		return -ENOMEM;

	cdev_ctx->major = MAJOR(gctx.device_number);
	cdev_ctx->minor = alloc_minor(&gctx);
	if (cdev_ctx->minor == IFC_MCDMA_DEV_MAX) {
		pr_err("Failed to create char dev node\n");
		goto fail_cleanup;
	}

	cdev_ctx->qcsr = pctx->info[0].iaddr;
	cdev_ctx->pdev_ctx = dev_ctx;
	strcpy(cdev_ctx->chr_dev_name, IFC_CHARDEV_NAME_PREFIX);
	strcat(cdev_ctx->chr_dev_name, dev_ctx->bdf);

	mutex_init(&cdev_ctx->chnl_lock);
	mutex_init(&cdev_ctx->tid_lock);

#ifdef IFC_MCDMA_KMEM
	/* create the memory pool */
	strcpy(pool_name, "ifc_dma_pool_");
	strcat(pool_name, dev_ctx->bdf);
	size = IFC_MCDMA_BUF_LIMIT;
	size = ((size + PAGE_SIZE - 1) / PAGE_SIZE) * PAGE_SIZE;

	cdev_ctx->dmem_pool = dma_pool_create(pool_name, &pctx->pdev->dev,
					     size, PAGE_SIZE, 0);
	if (cdev_ctx->dmem_pool == NULL) {
		pr_err("Creation of DMA mem pool failed\n");
		goto fail_cleanup;
	}
	cdev_ctx->dmem_pool_size = size;
#endif

	*dctx = cdev_ctx;
	ret = ifc_mcdma_node_create(*dctx);
	if (ret < 0) {
		pr_err("Failed to create char dev node\n");
		goto fail_cleanup;
	}

	return 0;
fail_cleanup:
	kfree(*dctx);
	*dctx = NULL;
	return ret;
}


/* will be called from the pcie remove */
void ifc_device_chr_exit(struct ifc_chr_dev_ctx *dctx)
{
	if (unlikely(dctx == NULL)) {
		pr_err("Invalid Device Context\n");
		return;
	}

	ifc_mcdma_node_destroy(dctx);
	if (dctx->dmem_pool)
		dma_pool_destroy(dctx->dmem_pool);
	dealloc_minor(&gctx, dctx->minor);
	kfree(dctx);
}


/* creates global context --> will be called from module init*/
int ifc_global_chr_init(void)
{
	int ret;

	gctx.gclass = class_create(THIS_MODULE, IFC_MCDMA_CLASS_NAME);
	if (IS_ERR(gctx.gclass)) {
		pr_err("Failed to create driver class\n");
		return 0 - PTR_ERR(gctx.gclass);
	}

	ret = alloc_chrdev_region(&gctx.device_number, 0, IFC_MCDMA_DEV_MAX,
				  IFC_MCDMA_DRIVER_NAME);
	if (ret < 0) {
		pr_err("Failed to allocate global chr context\n");
		return -1;
	}
	return 0;
}

/* must be called before module exits */
void ifc_global_chr_exit(void)
{
	class_destroy(gctx.gclass);
	gctx.gclass = NULL;
	unregister_chrdev_region(gctx.device_number, IFC_MCDMA_DEV_MAX);
}
