// SPDX-License-Identifier: GPL-2.0
/*-
 * Copyright(c) 2020-21 Intel Corporation. All rights reserved.
 */
#ifdef DEBUG
char *intel_dbg_message = "WARNING: this is a DEBUG driver";
#define __INTEL__USE_DBG_CHK() asm("" :: "m"(intel_dbg_message))

#else /* DEBUG */

char *intel_release_message = "Intel PRODUCTION driver";
#define __INTEL__USE_DBG_CHK() asm("" :: "m"(intel_release_message))

#endif /* DEBUG */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/version.h>
#include <linux/list.h>
#include <linux/errno.h>
#include "ifc_mcdma_pci.h"
#include "ifc_mcdma_chr.h"
#include <mcdma_ip_params.h>
#ifndef PCI_VENDOR_ID_REDHAT_QUMRANET
#define PCI_VENDOR_ID_REDHAT_QUMRANET	0x1af4
#endif

#ifdef IFC_QDMA_MSI_ENABLE
#define IFC_MCDMA_BAS_EN
#define HAVE_ALLOC_IRQ_VECTORS
#endif

struct ifc_mcdma_global_ctx gctx;

static const char ifc_mcdma_driver_version[] = DRV_VERSION;
static const char ifc_mcdma_driver_string[] = DRV_SUMMARY;
static const char ifc_mcdma_copyright[] = "Copyright (c) 2020-21, Intel Corporation.";


MODULE_AUTHOR("Intel Corporation, <linux.nics@intel.com>");
MODULE_DESCRIPTION(DRV_SUMMARY);
MODULE_LICENSE("GPL");
MODULE_VERSION(DRV_VERSION);

static ssize_t
mcdma_dummy_show(struct device *dev, struct device_attribute *attr, char *buf)
{
	if (buf == NULL)
		return -1;

	return snprintf(buf, 30, "mcdma dummy message\n");
}


static ssize_t
mcdma_dummy_store(struct device *dev, struct device_attribute *attr,
		  const char *buf, size_t count)
{
	if (buf == NULL)
		return -1;
	return count;
}

static int parse_long_int(const char *arg)
{
        unsigned long arg_val;
        const char *c = arg;

        while(*c != '\0') {
                if ((*c < 48 || *c > 57) && (*c != 10))
                        return -1;
                c++;
        }
        sscanf(arg, "%lu", &arg_val);
        return arg_val;
}

#ifdef IFC_MCDMA_KMEM
static ssize_t
mcdma_payload_show(struct device *dev, struct device_attribute *attr, char *buf)
{
	struct ifc_pci_dev *pctx = dev_get_drvdata(dev);
	struct ifc_mcdma_dev_ctx *dev_ctx;

	if (buf == NULL)
		return -1;

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -1;
	}

	dev_ctx = container_of(pctx, struct ifc_mcdma_dev_ctx, pctx);
	return snprintf(buf, 30, "%lu\n", dev_ctx->chr_ctx->dmem_pool_size);
}

static ssize_t
mcdma_payload_store(struct device *dev, struct device_attribute *attr,
		  const char *buf, size_t count)
{
	struct ifc_pci_dev *pctx = dev_get_drvdata(dev);
	struct ifc_mcdma_dev_ctx *dev_ctx;
	unsigned long payload;
	size_t size;

	if (buf == NULL)
		return -1;

	payload = parse_long_int(buf);

	if (payload < 0 || payload > IFC_MCDMA_BUF_LIMIT)
		return -1;

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -1;
	}

	dev_ctx = container_of(pctx, struct ifc_mcdma_dev_ctx, pctx);

	if (dev_ctx->chr_ctx->dmem_pool)
		dma_pool_destroy(dev_ctx->chr_ctx->dmem_pool);

	/* create the memory pool */
	size = payload;
	size = ((size + PAGE_SIZE - 1) / PAGE_SIZE) * PAGE_SIZE;

	dev_ctx->chr_ctx->dmem_pool = dma_pool_create("ifc_dma_pool",
						&dev_ctx->pctx.pdev->dev,
						size, PAGE_SIZE, 0);
	if (dev_ctx->chr_ctx->dmem_pool == NULL) {
		pr_err("Creation of DMA mem pool failed\n");
		return -EINVAL;
	}
	dev_ctx->chr_ctx->dmem_pool_size = size;
	return count;
}
static
DEVICE_ATTR(mcdma_payload, S_IRUGO | S_IWUSR, mcdma_payload_show,
	    mcdma_payload_store);
#endif

static ssize_t
mcdma_ipreset_show(struct device *dev, struct device_attribute *attr, char *buf)
{
	struct ifc_pci_dev *pctx = dev_get_drvdata(dev);
	struct ifc_mcdma_dev_ctx *dev_ctx;

	if (buf == NULL)
		return -1;

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -1;
	}

	dev_ctx = container_of(pctx, struct ifc_mcdma_dev_ctx, pctx);
	return snprintf(buf, 30, "%s\n", dev_ctx->bdf);
}

static ssize_t
mcdma_ipreset_store(struct device *dev, struct device_attribute *attr,
		  const char *buf, size_t count)
{
	struct ifc_pci_dev *pctx = dev_get_drvdata(dev);
	struct ifc_mcdma_dev_ctx *dev_ctx;
	int val;

	if (buf == NULL)
		return -1;

	val = parse_long_int(buf);

	if (val != 1)
		return -1;

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -1;
	}

	dev_ctx = container_of(pctx, struct ifc_mcdma_dev_ctx, pctx);

	if (ifc_mcdma_ip_reset(dev_ctx->chr_ctx) < 0) {
		pr_err("Failed to reset device\n");
		return -1;
	}
	return count;
}

static ssize_t
mcdma_config_show(struct device *dev, struct device_attribute *attr, char *buf)
{
	struct ifc_pci_dev *pctx = dev_get_drvdata(dev);
	struct ifc_mcdma_dev_ctx *dev_ctx;
	uint32_t val;
	int retval;

	if (buf == NULL)
		return -1;

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -1;
	}

	dev_ctx = container_of(pctx, struct ifc_mcdma_dev_ctx, pctx);
	retval = pci_read_config_dword(pctx->pdev, dev_ctx->cfg_addr, &val);
	if (retval < 0)
		return retval;
	return snprintf(buf, 30, "0x%x 0x%x\n", dev_ctx->cfg_addr, val);
}

static ssize_t
mcdma_config_store(struct device *dev, struct device_attribute *attr,
		  const char *buf, size_t count)
{
	struct ifc_pci_dev *pctx = dev_get_drvdata(dev);
	struct ifc_mcdma_dev_ctx *dev_ctx;
	int val;

	if (buf == NULL)
		return -1;

	val = parse_long_int(buf);

	if (unlikely((val % sizeof(uint32_t)) != 0))
	{
		pr_err("config access is misaligned.");
		return -1;
	}

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -1;
	}

	dev_ctx = container_of(pctx, struct ifc_mcdma_dev_ctx, pctx);
	dev_ctx->cfg_addr = val;

	return count;
}

static
DEVICE_ATTR(mcdma_ipreset, S_IRUGO | S_IWUSR, mcdma_ipreset_show,
	    mcdma_ipreset_store);
static
DEVICE_ATTR(mcdma_dummy, S_IRUGO | S_IWUSR, mcdma_dummy_show,
	    mcdma_dummy_store);
static
DEVICE_ATTR(mcdma_config, S_IRUGO | S_IWUSR, mcdma_config_show,
	    mcdma_config_store);

static struct attribute *dev_attrs[] = {
	&dev_attr_mcdma_dummy.attr,
#ifdef IFC_MCDMA_KMEM
	&dev_attr_mcdma_payload.attr,
#endif
	&dev_attr_mcdma_ipreset.attr,
	&dev_attr_mcdma_config.attr,
	NULL,
};

static const struct attribute_group dev_attr_grp = {
	.attrs = dev_attrs,
};

#ifdef IFC_QDMA_MSIX_ENABLE
static int
ifc_pci_enable_interrupts(struct pci_dev *pdev, struct ifc_msix_info *msix_info)
{
	struct msix_entry *table;
	int err = 0;
	int vectors;
	int i;

	if (unlikely(pdev == NULL || msix_info == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -EFAULT;
	}

	vectors = pci_msix_vec_count(pdev);
	if (vectors < 0) {
		pr_err("Failed to receive MSI-X vector count\n");
		return -1;
	}

	table = kcalloc(vectors, sizeof(struct msix_entry), GFP_KERNEL);
	if (table == NULL) {
		pr_err("Failed to allocated memory\n");
		return -ENOMEM;
	}

	for (i = 0; i < vectors; i++)
		table[i].entry = i;

#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 8, 0)
	if (pci_enable_msix(pdev, table, vectors) != 0) {
#else
	if (pci_enable_msix_exact(pdev, table, vectors) != 0) {
#endif
		pr_err("failed while enabling interrupts %d\n", err);
		goto fail_cleanup;
	}
	pr_debug("Successfully enabled PCI MSI-X: vectors: %d", vectors);
	msix_info->nvectors = vectors;
	msix_info->table = table;
	spin_lock_init(&msix_info->irq_lock);
	return 0;

fail_cleanup:
	kfree(table);
	return -1;
}

static void
ifc_pci_disable_interrupts(struct pci_dev *pdev,
			   struct ifc_msix_info *msix_info)
{
	if (unlikely(pdev == NULL || msix_info == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return;
	}
	pci_disable_msix(pdev);
	kfree(msix_info->table);
}
#elif defined(IFC_QDMA_MSI_ENABLE)
static int
ifc_pci_enable_msi_interrupts(struct pci_dev *pdev)
{
#ifndef HAVE_ALLOC_IRQ_VECTORS
	if (pci_enable_msi(pdev) == 0) {
		pr_err("using MSI");
		return 0;
	}
#else
	if (pci_alloc_irq_vectors(pdev, 2, 3, PCI_IRQ_MSI) >= 1) {
		pr_err("using MSI");
		return 0;
	}
#endif
	return -1;
}

static void ifc_pci_disable_msi_interrupts(struct pci_dev *pdev)
{
#ifndef HAVE_ALLOC_IRQ_VECTORS
	pci_disable_msi(pdev);
#else
	pci_free_irq_vectors(pdev);
#endif
}
#endif

static int
ifc_pci_setup_iomem(struct pci_dev *dev, struct ifc_mcdma_bar_info *info,
		       int pci_bar, const char *name)
{
	unsigned long addr, len;
	void *internal_addr = NULL;

	if (unlikely(dev == NULL || info == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -EFAULT;
	}

	addr = pci_resource_start(dev, pci_bar);
	len = pci_resource_len(dev, pci_bar);
	if (addr == 0 || len == 0)
		return -EINVAL;

	internal_addr = ioremap(addr, len);
	if (IS_ERR(internal_addr))
		return PTR_ERR(internal_addr);


	info->name = name;
	info->iaddr = internal_addr;
	info->size = len;

	return 0;
}

static int
ifc_pci_setup_ioport(struct pci_dev *dev, struct ifc_mcdma_bar_info *info,
			int pci_bar, const char *name)
{
	unsigned long addr, len;

	if (unlikely(dev == NULL || info == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -EFAULT;
	}
	addr = pci_resource_start(dev, pci_bar);
	len = pci_resource_len(dev, pci_bar);
	if (addr == 0 || len == 0)
		return -EINVAL;

	info->name = name;
	info->size = len;
	return 0;
}

/* Unmap previously ioremap'd resources */
static void
ifc_pci_release_iomem(struct ifc_mcdma_bar_info *info, int size)
{
	int i;

	if (unlikely(info == NULL)) {
		pr_err("Invalid BAR Context\n");
		return;
	}

	size = size ? size : PCI_STD_RESOURCE_END + 1;
	for (i = 0; i < size; i++) {
		if (info[i].iaddr)
			iounmap(info[i].iaddr);
	}
}

static int
ifc_setup_bars(struct pci_dev *dev, struct ifc_mcdma_bar_info **info)
{
	int i, iom, iop, ret;
	unsigned long flags;
	static const char *bar_names[PCI_STD_RESOURCE_END + 1]  = {
		"BAR0",
		"BAR1",
		"BAR2",
		"BAR3",
		"BAR4",
		"BAR5",
	};

	if (unlikely(dev == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -EFAULT;
	}

	iom = 0;
	iop = 0;

	*info = kzalloc(ARRAY_SIZE(bar_names) *
			sizeof(struct ifc_mcdma_bar_info), GFP_KERNEL);
	if (*info == NULL)
		return -ENOMEM;

	for (i = 0; i < ARRAY_SIZE(bar_names); i++) {
		if (pci_resource_len(dev, i) != 0 &&
		    pci_resource_start(dev, i) != 0) {
			flags = pci_resource_flags(dev, i);
			if (flags & IORESOURCE_MEM) {
				ret = ifc_pci_setup_iomem(dev, *info + i, i,
							  bar_names[i]);
				if (ret != 0)
					goto fail_cleanup;
				iom++;
			} else if (flags & IORESOURCE_IO) {
				ret = ifc_pci_setup_ioport(dev, *info + i,
							   i, bar_names[i]);
				if (ret != 0)
					goto fail_cleanup;

				iop++;
			}
		}
	}

	if (iom != 0 || iop != 0)
		return 0;
fail_cleanup:
	ifc_pci_release_iomem(*info, i);
	kfree(*info);
	*info = NULL;
	return -1;
}

static int
ifc_mcdma_pci_init(struct ifc_pci_dev *pctx,
	  struct pci_dev *dev, const struct pci_device_id *id)
{
	int err;

	if (unlikely(dev == NULL || pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return -EFAULT;
	}

#ifdef HAVE_PCI_IS_BRIDGE_API
	if (pci_is_bridge(dev)) {
		dev_warn(&dev->dev, "Ignoring PCI bridge device\n");
		return -ENODEV;
	}
#endif
	/*
	 * enable device: ask low-level code to enable I/O and
	 * memory
	 */
	err = pci_enable_device(dev);
	if (err != 0) {
		dev_err(&dev->dev, "Cannot enable PCI device\n");
		goto fail_free;
	}


	/* enable bus mastering on the device */
	pci_set_master(dev);
	pctx->pdev = dev;

	/* remap IO memory */
	err = ifc_setup_bars(dev, &pctx->info);
	if (err != 0) {
		pr_err("Bar Mapping Failed\n");
		goto fail_disable_pcidev;
	}
	pr_debug("Bar Mapping Successful\n");

	if (dma_set_mask_and_coherent(&dev->dev, DMA_BIT_MASK(64))) {
		dev_err(&dev->dev, "Cannot set DMA mask\n");
		goto fail_release_barinfo;
	}

	/* create /sysfs entry */
	err = sysfs_create_group(&dev->dev.kobj, &dev_attr_grp);
	if (err != 0)
		goto fail_release_barinfo;

	pci_set_drvdata(dev, pctx);

	return 0;

fail_release_barinfo:
	ifc_pci_release_iomem(pctx->info, 0);
	kfree(pctx->info);
	pctx->info = NULL;
fail_disable_pcidev:
	pci_disable_device(dev);
fail_free:
	kfree(pctx);
	return err;
}

static void ifc_mcdma_pci_free(struct pci_dev *dev)
{
	struct ifc_pci_dev *pctx = pci_get_drvdata(dev);

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return;
	}
	ifc_pci_release_iomem(pctx->info, 0);
	kfree(pctx->info);
	pctx->info = NULL;

	sysfs_remove_group(&dev->dev.kobj, &dev_attr_grp);
	pci_disable_device(dev);
	pci_set_drvdata(dev, NULL);
}


#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 8, 0)
static int __devinit
#else
static int
#endif
ifc_pci_probe(struct pci_dev *dev, const struct pci_device_id *id)
{
	struct ifc_mcdma_dev_ctx *dev_ctx;
	int err;

	/* Create device context */
	dev_ctx = kzalloc(sizeof(struct ifc_mcdma_dev_ctx), GFP_KERNEL);
	if (!dev_ctx)
		return -ENOMEM;

	INIT_LIST_HEAD(&dev_ctx->list);

	/* Complete PCI configurations */
	err = ifc_mcdma_pci_init(&dev_ctx->pctx, dev, id);
	if (err < 0)
		goto fail_cleanup;

	strncpy(dev_ctx->bdf, dev_name(&dev->dev), 18);
#ifdef IFC_QDMA_MSIX_ENABLE
	/* Setup msix support */
	err = ifc_pci_enable_interrupts(dev, &dev_ctx->msix_info);
	if (err < 0) {
		pr_err("Failed to enable interrupt\n");
		goto fail_pci_free;
	}
	pr_debug("Successfully enabled interrupts\n");
#elif defined(IFC_QDMA_MSI_ENABLE)
	if(ifc_pci_enable_msi_interrupts(dev) < 0){
		pr_err("failed to enale MSI\n");
		return -1;
	}
#endif
	/* Initialize character device context */
	err = ifc_device_chr_init(&dev_ctx->chr_ctx, dev_ctx);
	if (err < 0)
		goto fail_pci_disable_intr;

	/* Insert it into the device list */
	mutex_lock(&gctx.global_lock);
	list_add_tail(&dev_ctx->list, &gctx.dev_list);
	mutex_unlock(&gctx.global_lock);

	atomic_inc(&gctx.dev_counter);

	pr_debug("PCIe device initialization completed\n");
	return 0;

fail_pci_disable_intr:
#ifdef IFC_QDMA_MSIX_ENABLE
	ifc_pci_disable_interrupts(dev, &dev_ctx->msix_info);
fail_pci_free:
	ifc_mcdma_pci_free(dev);
#endif
	kfree(dev_ctx->pctx.info);
fail_cleanup:
	kfree(dev_ctx);
	return err;
}

static void
ifc_pci_remove(struct pci_dev *dev)
{
	struct ifc_pci_dev *pctx = pci_get_drvdata(dev);
	struct ifc_mcdma_dev_ctx *dev_ctx;

	if (unlikely(pctx == NULL)) {
		pr_err("Invalid PCI device Context\n");
		return;
	}

	dev_ctx = container_of(pctx, struct ifc_mcdma_dev_ctx, pctx);

	/* Free character device context */
	ifc_device_chr_exit(dev_ctx->chr_ctx);

	dev_ctx->chr_ctx = NULL;

#ifdef IFC_QDMA_MSIX_ENABLE
	ifc_pci_disable_interrupts(dev, &dev_ctx->msix_info);
#elif defined(IFC_MCDMA_BAS_EN)
	ifc_pci_disable_msi_interrupts(dev);
#endif
	ifc_mcdma_pci_free(dev);

	mutex_lock(&gctx.global_lock);
	list_del(&dev_ctx->list);
	mutex_unlock(&gctx.global_lock);

	kfree(dev_ctx);
	atomic_dec(&gctx.dev_counter);
	pr_debug("Device Exit Successful\n");
}


/**
 * ifc_io_error_detected - called when PCI error is detected
 * @dev: Pointer to PCI device
 * @state: The current pci connection state
 *
 * This function is called after a PCI bus error affecting
 * this device has been detected.
 */
static pci_ers_result_t ifc_io_error_detected(struct pci_dev *dev,
					      pci_channel_state_t state)
{
	pr_err("PCI error occured: channel state: %u\n", state);
	switch (state) {
	case pci_channel_io_normal:
		/* Non Correctable Non-Fatal errors */
		return PCI_ERS_RESULT_CAN_RECOVER;
	case pci_channel_io_perm_failure:
		return PCI_ERS_RESULT_DISCONNECT;
	case pci_channel_io_frozen:
		pci_disable_device(dev);
		return PCI_ERS_RESULT_NEED_RESET;
	default:
		pr_err("default error\n");
	}

	return PCI_ERS_RESULT_NEED_RESET;
}


/**
 * ifc_io_slot_reset - called after the pci bus has been reset.
 * @dev: Pointer to PCI device
 *
 * Restart the card from scratch, as if from a cold-boot. Implementation
 * may resemble the first half of the ifc_resume routine.
 */
static pci_ers_result_t ifc_io_slot_reset(struct pci_dev *dev)
{
	int err;

	err = pci_enable_device_mem(dev);
	if (err) {
		pr_err("Cannot re-enable PCI device after reset.\n");
		return PCI_ERS_RESULT_DISCONNECT;
	}

	pci_set_master(dev);

	pci_restore_state(dev);

	return PCI_ERS_RESULT_RECOVERED;
}

/**
 * ifc_io_resume - called when traffic can start flowing again.
 * @dev: Pointer to PCI device
 *
 * This callback is called when the error recovery drivers tells us that
 * its OK to resume normal operation. Implementation resembles the
 * second-half of the ifc_resume function.
 */
static void ifc_io_resume(struct pci_dev *dev)
{
	if (dev == NULL)
		return;

	pci_set_master(dev);
	pci_restore_state(dev);
}

static const struct pci_error_handlers ifc_err_handler = {
	.error_detected = ifc_io_error_detected,
	.slot_reset = ifc_io_slot_reset,
	.resume = ifc_io_resume
};

struct pci_device_id ifc_pci_tbl[] = {
	{ PCI_DEVICE(VENDOR_ID, DEVICE_ID) },
	{0, }
};

static struct pci_driver ifc_pci_driver = {
	.name = "ifc_mcdma_chardev",
	.id_table = ifc_pci_tbl,
	.probe = ifc_pci_probe,
	.remove = ifc_pci_remove,
	.err_handler = &ifc_err_handler
};

/// The default vendor ID that the driver support
static int vendor_id = VENDOR_ID;
module_param(vendor_id, int, S_IRUGO);
/// The default device ID that the driver support
static int device_id = DEVICE_ID;
module_param(device_id, int, S_IRUGO);

static int __init
ifc_pci_init_module(void)
{
	int err;

	pr_info("%s - version %s\n", ifc_mcdma_driver_string,
				     ifc_mcdma_driver_version);
	pr_info("%s\n", ifc_mcdma_copyright);
	__INTEL__USE_DBG_CHK();

	INIT_LIST_HEAD(&gctx.dev_list);
	atomic_set(&gctx.dev_counter, 0);
	mutex_init(&gctx.global_lock);

	err = ifc_global_chr_init();
	if (err < 0)
		return -1;

	ifc_pci_tbl[0].vendor = vendor_id;
	ifc_pci_tbl[0].device = device_id;
	return pci_register_driver(&ifc_pci_driver);
}

static void __exit
ifc_pci_exit_module(void)
{
	struct ifc_mcdma_dev_ctx *dev_ctx, *n;

	pci_unregister_driver(&ifc_pci_driver);

	list_for_each_entry_safe(dev_ctx, n, &gctx.dev_list, list) {
		list_del(&dev_ctx->list);
		kfree(dev_ctx);
	}

	ifc_global_chr_exit();
	pr_debug("Driver Exit Successful\n");
}

module_init(ifc_pci_init_module);
module_exit(ifc_pci_exit_module);
